@@ -30,25 +30,43 @@ using System.Collections;
 using System.IO;
 using System.Net;
 
+using Beagle.Util;
 using Beagle.Daemon;
 
 class ExtractContentTool {
 
+	static bool tokenize = false;
+
+	static void DisplayContent (string line)
+	{
+		if (tokenize) {
+			
+			string [] parts = line.Split (' ');
+			for (int i = 0; i < parts.Length; ++i) {
+				string part = parts [i].Trim ();
+				if (part != "")
+					Console.WriteLine ("{0}", part);
+			}
+
+		} else {
+			Console.WriteLine (line);
+		}
+	}
+
 	static void Main (String[] args)
 	{
 		bool firstArg = true;
 
 		foreach (String arg in args) {
+
+			if (arg == "--tokenize") {
+				tokenize = true;
+				continue;
+			}
 			
 			FilteredIndexable indexable;
 
-			string uri = arg;
-
-			if (!uri.StartsWith ("file://")) {
-				uri = Path.GetFullPath (uri);
-				uri = "file://" + uri;
-			}
-				
+			Uri uri = UriFu.PathToFileUri (arg);
 			Console.WriteLine ("uri: {0}", uri);
 
 			indexable = new FilteredIndexable (uri);
@@ -60,6 +78,11 @@ class ExtractContentTool {
 			}
 			firstArg = false;
 
+			// FIX: We should call "Build" as it updates the 
+			// "Flavor" and "Filter" members, failing which 
+			// will result in a "No filter" situation. :)
+			indexable.Build ();
+
 			Console.WriteLine ("Filename: " + uri);
 			Console.WriteLine ("  Flavor: " + indexable.Flavor);
 			if (! indexable.HaveFilter)
@@ -91,10 +114,12 @@ class ExtractContentTool {
 						Console.WriteLine ("Content:");
 						first = false;
 					}
-					Console.WriteLine (line);
+					DisplayContent (line);
 				}
 
-				if (! first)
+				if (first)
+					Console.WriteLine ("(no content)");
+				else
 					Console.WriteLine ();
 			}
 
@@ -107,10 +132,12 @@ class ExtractContentTool {
 						Console.WriteLine ("HotContent:");
 						first = false;
 					}
-					Console.WriteLine (line);
+					DisplayContent (line);
 				}
 
-				if (! first)
+				if (first)
+					Console.WriteLine ("(no hot content)");
+				else
 					Console.WriteLine ();
 			}
 		}
