@@ -31,7 +31,7 @@ using System.IO;
 using System.Text;
 using System.Reflection;
 
-using BU = Beagle.Util;
+using Beagle.Util;
 
 namespace Beagle.Daemon {
 
@@ -73,6 +73,19 @@ namespace Beagle.Daemon {
 
 		//////////////////////////
 
+		private bool crawlMode = false;
+
+		public void EnableCrawlMode ()
+		{
+			crawlMode = true;
+		}
+		
+		protected bool CrawlMode {
+			get { return crawlMode; }
+		}
+
+		//////////////////////////
+
 		int hotCount = 0;
 		int freezeCount = 0;
 
@@ -108,6 +121,22 @@ namespace Beagle.Daemon {
 			get { return freezeCount > 0; }
 		}
 
+		//////////////////////////
+		
+		private bool snippetMode = false;
+		private TextWriter snippetWriter = null;
+
+		public bool SnippetMode {
+			get { return snippetMode; }
+			set { snippetMode = value; }
+		}
+		
+		public void AttachSnippetWriter (TextWriter writer)
+		{
+			if (snippetMode)
+				snippetWriter = writer;
+		}
+
 		//////////////////////////
 
 		private ArrayList textPool;
@@ -116,17 +145,24 @@ namespace Beagle.Daemon {
 
 		public void AppendText (string str)
 		{
+			//Logger.Log.Debug ("AppendText (\"{0}\")", str);
 			if (! IsFrozen && str != null && str != "") {
+
+				// FIXME: We should be smarter about newlines
+				str = str.Replace ("\n", " ");
+
 				textPool.Add (str);
 				if (IsHot)
 					hotPool.Add (str);
+				if (snippetWriter != null)
+					snippetWriter.Write (str);
 			}
 		}
 
 		private bool NeedsWhiteSpace (ArrayList array)
 		{
 			if (array.Count == 0)
-				return false;
+				return true;
 			
 			string last = (string) array [array.Count-1];
 			if (last.Length > 0
@@ -138,8 +174,12 @@ namespace Beagle.Daemon {
 
 		public void AppendWhiteSpace ()
 		{
-			if (NeedsWhiteSpace (textPool))
+			//Logger.Log.Debug ("AppendWhiteSpace ()");
+			if (NeedsWhiteSpace (textPool)) {
 				textPool.Add (" ");
+				if (snippetWriter != null)
+					snippetWriter.Write (" ");
+			}
 			if (NeedsWhiteSpace (hotPool))
 				hotPool.Add (" ");
 			
@@ -151,6 +191,12 @@ namespace Beagle.Daemon {
 				propertyPool.Add (prop);
 		}
 
+		public void AddStructuralBreak ()
+		{
+			if (snippetWriter != null)
+				snippetWriter.WriteLine ();
+		}
+
 		//////////////////////////
 
 		private bool isFinished = false;
@@ -193,8 +239,10 @@ namespace Beagle.Daemon {
 		*/
 
 		private bool isOpen = false;
-		private FileInfo currentInfo;
 		private string tempFile = null;
+		private FileInfo currentInfo = null;
+		private FileStream currentStream = null;
+		private StreamReader currentReader = null;
 
 		public void Open (Stream stream)
 		{
@@ -223,10 +271,24 @@ namespace Beagle.Daemon {
 			propertyPool = new ArrayList ();
 
 			currentInfo = info;
-			
+
+			// Open a stream for this file.
+			currentStream = new FileStream (info.FullName,
+							FileMode.Open,
+							FileAccess.Read,
+							FileShare.Read);
+
+			// Our default assumption is sequential reads.
+			// FIXME: Is this the right thing to do here?
+			FileAdvise.IncreaseReadAhead (currentStream);
+
+			// Give the OS a hint that we will be reading this
+			// file soon.
+			FileAdvise.PreLoad (currentStream);			
+
 			DoOpen (info);
 			isOpen = true;
-			
+
 			if (IsFinished) {
 				isOpen = false;
 				return;
@@ -238,6 +300,15 @@ namespace Beagle.Daemon {
 				return;
 			}
 
+			// Close and reset our TextReader
+			if (currentReader != null) {
+				currentReader.Close ();
+				currentReader = null;
+			}
+
+			// Seek back to the beginning of our stream
+			currentStream.Seek (0, SeekOrigin.Begin);
+
 			DoPullSetup ();
 			if (IsFinished) {
 				isOpen = false;
@@ -245,15 +316,29 @@ namespace Beagle.Daemon {
 			}
 		}
 
-		public FileInfo CurrentFileInfo {
+		public FileInfo FileInfo {
 			get { return currentInfo; }
 		}
 
-		private bool Pull () 
+		public Stream Stream {
+			get { return currentStream; }
+		}
+
+		public TextReader TextReader {
+			get {
+				if (currentReader == null
+				    && currentStream != null) {
+					currentReader = new StreamReader (currentStream);
+				}
+
+				return currentReader;
+			}
+		}
+
+		private bool Pull ()
 		{
 			if (IsFinished) {
-				DoClose ();
-				Cleanup ();
+				Close ();
 				return false;
 			}
 
@@ -262,10 +347,32 @@ namespace Beagle.Daemon {
 			return true;
 		}
 
-		public void Cleanup ()
+		private bool closed = false;
+
+		private void Close ()
 		{
+			if (currentStream == null)
+				return;
+
+			DoClose ();
+
+			// When crawling, give the OS a hint that we don't
+			// need to keep this file around in the page cache.
+			if (CrawlMode)
+				FileAdvise.FlushCache (currentStream);
+
+			if (currentReader != null)
+				currentReader.Close ();
+
+			currentStream.Close ();
+			currentStream = null;
+
+			if (snippetWriter != null)
+				snippetWriter.Close ();
+
 			if (tempFile != null)
 				File.Delete (tempFile);
+
 		}
 
 		private string PullFromArray (ArrayList array)
@@ -291,12 +398,12 @@ namespace Beagle.Daemon {
 
 		public TextReader GetTextReader ()
 		{
-			return new BU.PullingReader (new BU.PullingReader.Pull (PullText));
+			return new PullingReader (new PullingReader.Pull (PullText));
 		}
 
 		public TextReader GetHotTextReader ()
 		{
-			return new BU.PullingReader (new BU.PullingReader.Pull (PullHotText));
+			return new PullingReader (new PullingReader.Pull (PullHotText));
 		}
 
 		public IEnumerable Properties {
@@ -312,7 +419,7 @@ namespace Beagle.Daemon {
 			int count = 0;
 
 			foreach (Type t in assembly.GetTypes ()) {
-				if (t.IsSubclassOf (typeof (Filter))) {
+				if (t.IsSubclassOf (typeof (Filter)) && ! t.IsAbstract) {
 					Filter filter = (Filter) Activator.CreateInstance (t);
 					bool first = true;
 					foreach (Flavor flavor in filter.SupportedFlavors) {
@@ -341,7 +448,7 @@ namespace Beagle.Daemon {
 				return;
 
 			if (! Directory.Exists (dir)) {
-				//Console.WriteLine ("'{0}' is not a directory: No filters loaded", dir);
+				//Logger.Log.Debug ("'{0}' is not a directory: No filters loaded", dir);
 				return;
 			}
 
@@ -350,7 +457,7 @@ namespace Beagle.Daemon {
 				if (file.Extension == ".dll") {
 					Assembly a = Assembly.LoadFrom (file.FullName);
 					int n = ScanAssemblyForFilters (a);
-					//Console.WriteLine ("Loaded {0} filters from {1}", n, file.FullName);
+					//Logger.Log.Debug ("Loaded {0} filters from {1}", n, file.FullName);
 				}
 			}
 		}
@@ -364,7 +471,7 @@ namespace Beagle.Daemon {
 			else if (path [path.Length-1] == ':')
 				path += PathFinder.FilterDir;
 
-			foreach (string dir in path.Split (':')) 
+			foreach (string dir in path.Split (':'))
 				FindAssemblies (dir);
 		}
 
